package org.jutils.jdbc;

/**
 * <p>------------------------------------------------------------------------------<br>
 * Name:     	SqlClauseFactory<br>
 * Project:   	jutils<br>
 * Copyright: 	jutils.org, see license file<br>
 * Comment:		Provides an easy-to-use- Singleton for creating SQL Query strings.<br>
 * Creation date: (07.07.2000 17:44:33)   <br>
 * Version:   <pre>$Id: SqlClauseFactory.java,v 1.1 2000/07/13 08:43:15 winkler Exp $</pre> <br>
 * ------------------------------------------------------------------------------<p>
 * @author: M.Winkler, doubleSlash GmbH
 */
import org.jutils.*;
import java.sql.*;
public class SqlClauseFactory implements CharConstants {
   public static final String QUERY_COMMAND = "SELECT";
   public static final String FROM_CLAUSE = "FROM";
   public static final String WHERE_CLAUSE = "WHERE";
   public static final String FOR_CLAUSE = "FOR";

   /** The separator to be used to separate distinct columns. */
   public static final char COLUMN_SEPARATOR = ',';
   public static final char TABLE_SEPARATOR = ',';
   public static final char EXPRESSION_SEPARATOR = ',';
   public static final String SENTENCE_TERMINATOR = ";";

   /**
    * Composes a SQL Select statement, according to the provided values.
    * Creation date: (07.07.2000 17:49:46)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @param table java.lang.String [] The name of the tables to be accessed.
    * @param columns java.lang.String[] The names of the columns to be accessed.
    * @param expressions java.lang.String[]
    * @param modifiers java.lang.String [] a modifier (one of the constants).
    */
   public static StringBuffer composeQuery(String[] tables, String[] columns, String[] expressions, StringBuffer dest) {

      // ---------------------------------------------------------------
      // initialize returnValue
      // ---------------------------------------------------------------
      StringBuffer returnValue = dest;
      if (returnValue == null)
         returnValue = new StringBuffer();
      else
         returnValue.setLength(0);

      // ---------------------------------------------------------------
      // create "SELECT "- section
      // ---------------------------------------------------------------
      returnValue.append(QUERY_COMMAND).append(SPACE);

      // insert column names (if present)
      if ((columns != null) && (columns.length > 0)) {
         for (int columnIndex = 0; columnIndex < columns.length; columnIndex++) {
            if (columnIndex > 0)
               returnValue.append(COLUMN_SEPARATOR);
            returnValue.append(columns[columnIndex]);
         }
      }
      else
         returnValue.append(ASTERISK);

      // ---------------------------------------------------------------
      // create "FROM" - section
      // ---------------------------------------------------------------
      returnValue.append(SPACE);
      returnValue.append(FROM_CLAUSE);
      returnValue.append(SPACE);
      for (int tableIndex = 0; tableIndex < tables.length; tableIndex++) {
         if (tableIndex > 0)
            returnValue.append(TABLE_SEPARATOR);
         returnValue.append(tables[tableIndex]);
      }

      // ---------------------------------------------------------------
      // create "WHERE" - section
      // ---------------------------------------------------------------
      if ((expressions != null) && (expressions.length > 0)) {
         returnValue.append(SPACE);
         returnValue.append(WHERE_CLAUSE);
         returnValue.append(SPACE);
         for (int expressionIndex = 0; expressionIndex < expressions.length; expressionIndex++) {
            if (expressionIndex > 0)
               returnValue.append(EXPRESSION_SEPARATOR);
            returnValue.append(expressions[expressionIndex]);
         }
      }

      // append terminator ;
      returnValue.append(SENTENCE_TERMINATOR);

      // give it back
      return returnValue;
   }

   /**
    * Creation date: (07.07.2000 17:49:46)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @param table java.lang.String The name of at least one the table to be accessed.
    * @param columns java.lang.String The names of columns to be accessed. May be null or "".
    * @param expressions java.lang.String The expressions to fit. May be null.
    * @param modifiers java.lang.String [] a modifier (one of the constants).
    */
   public static StringBuffer composeQuery(String tables, String columns, String expressions, StringBuffer dest) {

      // ---------------------------------------------------------------
      // initialize returnValue
      // ---------------------------------------------------------------
      StringBuffer returnValue = dest;
      if (returnValue == null)
         returnValue = new StringBuffer();
      else
         returnValue.setLength(0);


      // ---------------------------------------------------------------
      // create "SELECT "- section
      // ---------------------------------------------------------------
      returnValue.append(QUERY_COMMAND).append(SPACE);

      // insert column names (if present)
      if ((columns != null) && (columns.length() > 0)) {
         returnValue.append(columns);
      }
      else
         returnValue.append(ASTERISK);

      // ---------------------------------------------------------------
      // create "FROM" - section
      // ---------------------------------------------------------------
      returnValue.append(FROM_CLAUSE);
      returnValue.append(tables);

      // ---------------------------------------------------------------
      // create "WHERE" - section
      // ---------------------------------------------------------------
      if ((expressions != null) && (expressions.length() > 0)) {
         returnValue.append(WHERE_CLAUSE);
         returnValue.append(expressions);
      }

      // append terminator ;
      returnValue.append(SENTENCE_TERMINATOR);

      // give it back
      return returnValue;
   }
}
