package org.jutils.text;

/*------------------------------------------------------------------------------
Name:      StringHelper.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   Properties for jutils, see jutils.property
Version:   $Id: StringHelper.java,v 1.4 2000/07/13 12:26:42 ruff Exp $
------------------------------------------------------------------------------*/
/**
 * Some utilities to manipulate strings.
 * @author Marcel Ruff
 */
public class StringHelper
{
   private final static String ME = "StringHelper";

   /**
   * Replace exactly one occurrence of "from" with to "to"
   */
   public final static String replaceFirst(String str, String from, String to) {
      if (str == null || str.length() < 1 || from == null || to == null)
         return str;
      int index = str.indexOf(from);
      if (index >= 0) {
         StringBuffer tmp = new StringBuffer("");
         if (index > 0)
            tmp.append(str.substring(0, index));
         tmp.append(to);
         tmp.append(str.substring(index + from.length()));
         return tmp.toString();
      }
      else
         return str;
   }

   /**
   * Replace all occurrences of "from" with to "to".
   */
   public final static String replaceAll(String str, String from, String to) {
      if (str == null || str.length() < 1 || from == null || to == null)
         return str;
      if (str.indexOf(from) == -1)
         return str;
      StringBuffer buf = new StringBuffer("");
      String tail = str;
      while (true) {
         int index = tail.indexOf(from);
         if (index >= 0) {
            if (index > 0)
               buf.append(tail.substring(0, index));
            buf.append(to);
            tail = tail.substring(index + from.length());
         }
         else
            break;
      }
      buf.append(tail);
      return buf.toString();
   }

   /**
   * For testing only
   * <p />
   * java org.jutils.util.StringHelper
   */
   public static void main(String args[]) {
      String ME = "StringHelper";
      String input = "Hello my friend, my dear friend";
      System.out.println("Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "friend", "Jacqueline") + "'");
      System.out.println("Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "friend", "Jacqueline") + "'");
      input = "Hello my friend, my dear friend";
      System.out.println("Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "H", "") + "'");
      System.out.println("Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "H", "") + "'");
      input = "Hello my friend, my dear friend";
      System.out.println("Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, null, "") + "'");
      System.out.println("Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, null, "") + "'");
      input = "Hello my friend, my dear friend";
      System.out.println("Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "#", "Jacqueline") + "'");
      System.out.println("Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "#", "Jacqueline") + "'");
      input = "Hello my friend, my dear friend";
      System.out.println("Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "l", "&l;") + "'");
      System.out.println("Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "l", "&l;") + "'");
      input = " ";
      System.out.println("Converting '" + input + "' to (one replacement):\n" + "'" + StringHelper.replaceFirst(input, "l", "&l;") + "'");
      System.out.println("Converting '" + input + "' to (all replacement):\n" + "'" + StringHelper.replaceAll(input, "l", "&l;") + "'");
   }
}
